@@ -22,15 +22,19 @@
 import os
 import sys
 import argparse
+import pathlib
 from typing import Any, Dict, Iterator, List, Optional, Sequence, Tuple
 
+from PyQt5.QtCore import QLibraryInfo, QLocale
+
 from qutebrowser.config import config
 from qutebrowser.misc import objects
-from qutebrowser.utils import usertypes, qtutils, utils, log
+from qutebrowser.utils import usertypes, qtutils, utils, log, version
 
 
 _ENABLE_FEATURES = '--enable-features='
 _DISABLE_FEATURES = '--disable-features='
+_BLINK_SETTINGS = '--blink-settings='
 
 
 def qt_args(namespace: argparse.Namespace) -> List[str]:
@@ -57,36 +61,54 @@ def qt_args(namespace: argparse.Namespace) -> List[str]:
         assert objects.backend == usertypes.Backend.QtWebKit, objects.backend
         return argv
 
-    feature_prefixes = (_ENABLE_FEATURES, _DISABLE_FEATURES)
-    feature_flags = [flag for flag in argv if flag.startswith(feature_prefixes)]
-    argv = [flag for flag in argv if not flag.startswith(feature_prefixes)]
-    argv += list(_qtwebengine_args(namespace, feature_flags))
+    try:
+        # pylint: disable=unused-import
+        from qutebrowser.browser.webengine import webenginesettings
+    except ImportError:
+        # This code runs before a QApplication is available, so before
+        # backendproblem.py is run to actually inform the user of the missing
+        # backend. Thus, we could end up in a situation where we're here, but
+        # QtWebEngine isn't actually available.
+        # We shouldn't call _qtwebengine_args() in this case as it relies on
+        # QtWebEngine actually being importable, e.g. in
+        # version.qtwebengine_versions().
+        log.init.debug("QtWebEngine requested, but unavailable...")
+        return argv
+
+    special_prefixes = (_ENABLE_FEATURES, _DISABLE_FEATURES, _BLINK_SETTINGS)
+    special_flags = [flag for flag in argv if flag.startswith(special_prefixes)]
+    argv = [flag for flag in argv if not flag.startswith(special_prefixes)]
+    argv += list(_qtwebengine_args(namespace, special_flags))
 
     return argv
 
 
 def _qtwebengine_features(
-        feature_flags: Sequence[str],
+        versions: version.WebEngineVersions,
+        special_flags: Sequence[str],
 ) -> Tuple[Sequence[str], Sequence[str]]:
     """Get a tuple of --enable-features/--disable-features flags for QtWebEngine.
 
     Args:
-        feature_flags: Existing flags passed via the commandline.
+        versions: The WebEngineVersions to get flags for.
+        special_flags: Existing flags passed via the commandline.
     """
     enabled_features = []
     disabled_features = []
 
-    for flag in feature_flags:
+    for flag in special_flags:
         if flag.startswith(_ENABLE_FEATURES):
             flag = flag[len(_ENABLE_FEATURES):]
             enabled_features += flag.split(',')
         elif flag.startswith(_DISABLE_FEATURES):
             flag = flag[len(_DISABLE_FEATURES):]
             disabled_features += flag.split(',')
+        elif flag.startswith(_BLINK_SETTINGS):
+            pass
         else:
             raise utils.Unreachable(flag)
 
-    if qtutils.version_check('5.15', compiled=False) and utils.is_linux:
+    if versions.webengine >= utils.VersionNumber(5, 15, 1) and utils.is_linux:
         # Enable WebRTC PipeWire for screen capturing on Wayland.
         #
         # This is disabled in Chromium by default because of the "dialog hell":
@@ -98,8 +120,7 @@ def _qtwebengine_features(
         #
         # In theory this would be supported with Qt 5.13 already, but
         # QtWebEngine only started picking up PipeWire correctly with Qt
-        # 5.15.1. Checking for 5.15 here to pick up Archlinux' patched package
-        # as well.
+        # 5.15.1.
         #
         # This only should be enabled on Wayland, but it's too early to check
         # that, as we don't have a QApplication available at this point. Thus,
@@ -122,34 +143,98 @@ def _qtwebengine_features(
         if config.val.scrolling.bar == 'overlay':
             enabled_features.append('OverlayScrollbar')
 
-    if (qtutils.version_check('5.14', compiled=False) and
+    if (versions.webengine >= utils.VersionNumber(5, 14) and
             config.val.content.headers.referer == 'same-domain'):
         # Handling of reduced-referrer-granularity in Chromium 76+
         # https://chromium-review.googlesource.com/c/chromium/src/+/1572699
         #
         # Note that this is removed entirely (and apparently the default) starting with
-        # Chromium 89 (Qt 5.15.x or 6.x):
+        # Chromium 89 (presumably arriving with Qt 6.2):
         # https://chromium-review.googlesource.com/c/chromium/src/+/2545444
         enabled_features.append('ReducedReferrerGranularity')
 
-    if qtutils.version_check('5.15.2', compiled=False, exact=True):
+    if versions.webengine == utils.VersionNumber(5, 15, 2):
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-89740
-        # FIXME Not needed anymore with QtWebEngne 5.15.3 (or Qt 6), but we'll probably
-        # have no way to detect that...
         disabled_features.append('InstalledApp')
 
     return (enabled_features, disabled_features)
 
 
+def _get_locale_pak_path(locales_path: pathlib.Path, locale_name: str) -> pathlib.Path:
+    """Get the path for a locale .pak file."""
+    return locales_path / (locale_name + '.pak')
+
+
+def _get_pak_name(locale_name: str) -> str:
+    """Get the Chromium .pak name for a locale name.
+
+    Based on Chromium's behavior in l10n_util::CheckAndResolveLocale:
+    https://source.chromium.org/chromium/chromium/src/+/master:ui/base/l10n/l10n_util.cc;l=344-428;drc=43d5378f7f363dab9271ca37774c71176c9e7b69
+    """
+    if locale_name in {'en', 'en-PH', 'en-LR'}:
+        return 'en-US'
+    elif locale_name.startswith('en-'):
+        return 'en-GB'
+    elif locale_name.startswith('es-'):
+        return 'es-419'
+    elif locale_name == 'pt':
+        return 'pt-BR'
+    elif locale_name.startswith('pt-'):  # pragma: no cover
+        return 'pt-PT'
+    elif locale_name in {'zh-HK', 'zh-MO'}:
+        return 'zh-TW'
+    elif locale_name == 'zh' or locale_name.startswith('zh-'):
+        return 'zh-CN'
+
+    return locale_name.split('-')[0]
+
+
+def _get_lang_override(
+        webengine_version: utils.VersionNumber,
+        locale_name: str
+) -> Optional[str]:
+    """Get a --lang switch to override Qt's locale handling.
+
+    This is needed as a WORKAROUND for https://bugreports.qt.io/browse/QTBUG-91715
+    There is no fix yet, but we assume it'll be fixed with QtWebEngine 5.15.4.
+    """
+    if not config.val.qt.workarounds.locale:
+        return None
+
+    if webengine_version != utils.VersionNumber(5, 15, 3) or not utils.is_linux:
+        return None
+
+    locales_path = pathlib.Path(
+        QLibraryInfo.location(QLibraryInfo.TranslationsPath)) / 'qtwebengine_locales'
+    if not locales_path.exists():
+        log.init.debug(f"{locales_path} not found, skipping workaround!")
+        return None
+
+    pak_path = _get_locale_pak_path(locales_path, locale_name)
+    if pak_path.exists():
+        log.init.debug(f"Found {pak_path}, skipping workaround")
+        return None
+
+    pak_name = _get_pak_name(locale_name)
+    pak_path = _get_locale_pak_path(locales_path, pak_name)
+    if pak_path.exists():
+        log.init.debug(f"Found {pak_path}, applying workaround")
+        return pak_name
+
+    log.init.debug(f"Can't find pak in {locales_path} for {locale_name} or {pak_name}")
+    return 'en-US'
+
+
 def _qtwebengine_args(
         namespace: argparse.Namespace,
-        feature_flags: Sequence[str],
+        special_flags: Sequence[str],
 ) -> Iterator[str]:
     """Get the QtWebEngine arguments to use based on the config."""
-    is_qt_514 = (qtutils.version_check('5.14', compiled=False) and
-                 not qtutils.version_check('5.15', compiled=False))
+    versions = version.qtwebengine_versions(avoid_init=True)
 
-    if is_qt_514:
+    qt_514_ver = utils.VersionNumber(5, 14)
+    qt_515_ver = utils.VersionNumber(5, 15)
+    if qt_514_ver <= versions.webengine < qt_515_ver:
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-82105
         yield '--disable-shared-workers'
 
@@ -157,7 +242,7 @@ def _qtwebengine_args(
     # https://codereview.qt-project.org/c/qt/qtwebengine/+/256786
     # also see:
     # https://codereview.qt-project.org/c/qt/qtwebengine-chromium/+/265753
-    if qtutils.version_check('5.12.3', compiled=False):
+    if versions.webengine >= utils.VersionNumber(5, 12, 3):
         if 'stack' in namespace.debug_flags:
             # Only actually available in Qt 5.12.5, but let's save another
             # check, as passing the option won't hurt.
@@ -166,6 +251,13 @@ def _qtwebengine_args(
         if 'stack' not in namespace.debug_flags:
             yield '--disable-in-process-stack-traces'
 
+    lang_override = _get_lang_override(
+        webengine_version=versions.webengine,
+        locale_name=QLocale().bcp47Name(),
+    )
+    if lang_override is not None:
+        yield f'--lang={lang_override}'
+
     if 'chromium' in namespace.debug_flags:
         yield '--enable-logging'
         yield '--v=1'
@@ -174,20 +266,26 @@ def _qtwebengine_args(
         yield '--renderer-startup-dialog'
 
     from qutebrowser.browser.webengine import darkmode
-    blink_settings = list(darkmode.settings())
-    if blink_settings:
-        yield '--blink-settings=' + ','.join(f'{k}={v}' for k, v in blink_settings)
+    darkmode_settings = darkmode.settings(
+        versions=versions,
+        special_flags=special_flags,
+    )
+    for switch_name, values in darkmode_settings.items():
+        # If we need to use other switches (say, --enable-features), we might need to
+        # refactor this so values still get combined with existing ones.
+        assert switch_name in ['dark-mode-settings', 'blink-settings'], switch_name
+        yield f'--{switch_name}=' + ','.join(f'{k}={v}' for k, v in values)
 
-    enabled_features, disabled_features = _qtwebengine_features(feature_flags)
+    enabled_features, disabled_features = _qtwebengine_features(versions, special_flags)
     if enabled_features:
         yield _ENABLE_FEATURES + ','.join(enabled_features)
     if disabled_features:
         yield _DISABLE_FEATURES + ','.join(disabled_features)
 
-    yield from _qtwebengine_settings_args()
+    yield from _qtwebengine_settings_args(versions)
 
 
-def _qtwebengine_settings_args() -> Iterator[str]:
+def _qtwebengine_settings_args(versions: version.WebEngineVersions) -> Iterator[str]:
     settings: Dict[str, Dict[Any, Optional[str]]] = {
         'qt.force_software_rendering': {
             'software-opengl': None,
@@ -225,29 +323,30 @@ def _qtwebengine_settings_args() -> Iterator[str]:
             'always': None,
         }
     }
+    qt_514_ver = utils.VersionNumber(5, 14)
 
-    if (qtutils.version_check('5.14', compiled=False) and
-            not qtutils.version_check('5.15.2', compiled=False)):
+    if qt_514_ver <= versions.webengine < utils.VersionNumber(5, 15, 2):
         # In Qt 5.14 to 5.15.1, `--force-dark-mode` is used to set the
         # preferred colorscheme. In Qt 5.15.2, this is handled by a
         # blink-setting in browser/webengine/darkmode.py instead.
-        settings['colors.webpage.prefers_color_scheme_dark'] = {
-            True: '--force-dark-mode',
-            False: None,
+        settings['colors.webpage.preferred_color_scheme'] = {
+            'dark': '--force-dark-mode',
+            'light': None,
+            'auto': None,
         }
 
     referrer_setting = settings['content.headers.referer']
-    if qtutils.version_check('5.14', compiled=False):
+    if versions.webengine >= qt_514_ver:
         # Starting with Qt 5.14, this is handled via --enable-features
         referrer_setting['same-domain'] = None
     else:
         referrer_setting['same-domain'] = '--reduced-referrer-granularity'
 
-    can_override_referer = (
-        qtutils.version_check('5.12.4', compiled=False) and
-        not qtutils.version_check('5.13.0', compiled=False, exact=True)
-    )
     # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-60203
+    can_override_referer = (
+        versions.webengine >= utils.VersionNumber(5, 12, 4) and
+        versions.webengine != utils.VersionNumber(5, 13)
+    )
     referrer_setting['never'] = None if can_override_referer else '--no-referrers'
 
     for setting, args in sorted(settings.items()):
