@@ -25,7 +25,7 @@ from PyQt5.QtWidgets import QLabel
 
 from qutebrowser.config import configutils, configdata, configtypes, configexc
 from qutebrowser.utils import urlmatch, usertypes, qtutils
-from tests.helpers import utils
+from tests.helpers import testutils
 
 
 @pytest.fixture
@@ -277,7 +277,7 @@ def test_no_pattern_support(func, opt, pattern):
 
 
 def test_add_url_benchmark(values, benchmark):
-    blocked_hosts = list(utils.blocked_hosts())
+    blocked_hosts = list(testutils.blocked_hosts())
 
     def _add_blocked():
         for line in blocked_hosts:
@@ -294,7 +294,7 @@ def test_add_url_benchmark(values, benchmark):
 def test_domain_lookup_sparse_benchmark(url, values, benchmark):
     url = QUrl(url)
     values.add(False, urlmatch.UrlPattern("*.foo.bar.baz"))
-    for line in utils.blocked_hosts():
+    for line in testutils.blocked_hosts():
         values.add(False, urlmatch.UrlPattern(line))
 
     benchmark(lambda: values.get_for_url(url))
@@ -382,7 +382,7 @@ class TestFontFamilies:
         print(stylesheet)
         label.setStyleSheet(stylesheet)
 
-        with qtbot.waitExposed(label):
+        with qtbot.wait_exposed(label):
             # Needed so the font gets calculated
             label.show()
         info = label.fontInfo()
@@ -396,7 +396,7 @@ class TestFontFamilies:
         qtbot.add_widget(label)
         fallback_label.setText("fallback")
 
-        with qtbot.waitExposed(fallback_label):
+        with qtbot.wait_exposed(fallback_label):
             # Needed so the font gets calculated
             fallback_label.show()
 
