@@ -26,9 +26,9 @@ from unittest.mock import ANY, call
 
 import pytest
 
-from helpers import utils
+from helpers import testutils
 
-qute_lastpass = utils.import_userscript('qute-lastpass')
+qute_lastpass = testutils.import_userscript('qute-lastpass')
 
 default_lpass_match = [
     {
